# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from across.sdk.v1.models.alt_az_constraint import AltAzConstraint
from across.sdk.v1.models.earth_limb_constraint import EarthLimbConstraint
from across.sdk.v1.models.moon_angle_constraint import MoonAngleConstraint
from across.sdk.v1.models.saa_polygon_constraint import SAAPolygonConstraint
from across.sdk.v1.models.sun_angle_constraint import SunAngleConstraint
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

INSTRUMENTCONSTRAINTSINNER_ONE_OF_SCHEMAS = ["AltAzConstraint", "EarthLimbConstraint", "MoonAngleConstraint", "SAAPolygonConstraint", "SunAngleConstraint"]

class InstrumentConstraintsInner(BaseModel):
    """
    InstrumentConstraintsInner
    """
    # data type: EarthLimbConstraint
    oneof_schema_1_validator: Optional[EarthLimbConstraint] = None
    # data type: MoonAngleConstraint
    oneof_schema_2_validator: Optional[MoonAngleConstraint] = None
    # data type: SunAngleConstraint
    oneof_schema_3_validator: Optional[SunAngleConstraint] = None
    # data type: SAAPolygonConstraint
    oneof_schema_4_validator: Optional[SAAPolygonConstraint] = None
    # data type: AltAzConstraint
    oneof_schema_5_validator: Optional[AltAzConstraint] = None
    actual_instance: Optional[Union[AltAzConstraint, EarthLimbConstraint, MoonAngleConstraint, SAAPolygonConstraint, SunAngleConstraint]] = None
    one_of_schemas: Set[str] = { "AltAzConstraint", "EarthLimbConstraint", "MoonAngleConstraint", "SAAPolygonConstraint", "SunAngleConstraint" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = InstrumentConstraintsInner.model_construct()
        error_messages = []
        match = 0
        # validate data type: EarthLimbConstraint
        if not isinstance(v, EarthLimbConstraint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `EarthLimbConstraint`")
        else:
            match += 1
        # validate data type: MoonAngleConstraint
        if not isinstance(v, MoonAngleConstraint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `MoonAngleConstraint`")
        else:
            match += 1
        # validate data type: SunAngleConstraint
        if not isinstance(v, SunAngleConstraint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `SunAngleConstraint`")
        else:
            match += 1
        # validate data type: SAAPolygonConstraint
        if not isinstance(v, SAAPolygonConstraint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `SAAPolygonConstraint`")
        else:
            match += 1
        # validate data type: AltAzConstraint
        if not isinstance(v, AltAzConstraint):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AltAzConstraint`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in InstrumentConstraintsInner with oneOf schemas: AltAzConstraint, EarthLimbConstraint, MoonAngleConstraint, SAAPolygonConstraint, SunAngleConstraint. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in InstrumentConstraintsInner with oneOf schemas: AltAzConstraint, EarthLimbConstraint, MoonAngleConstraint, SAAPolygonConstraint, SunAngleConstraint. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("name")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `name` in the input.")

        # check if data type is `AltAzConstraint`
        if _data_type == "Altitude/Azimuth Avoidance":
            instance.actual_instance = AltAzConstraint.from_json(json_str)
            return instance

        # check if data type is `EarthLimbConstraint`
        if _data_type == "Earth Limb":
            instance.actual_instance = EarthLimbConstraint.from_json(json_str)
            return instance

        # check if data type is `MoonAngleConstraint`
        if _data_type == "Moon Angle":
            instance.actual_instance = MoonAngleConstraint.from_json(json_str)
            return instance

        # check if data type is `SAAPolygonConstraint`
        if _data_type == "South Atlantic Anomaly":
            instance.actual_instance = SAAPolygonConstraint.from_json(json_str)
            return instance

        # check if data type is `SunAngleConstraint`
        if _data_type == "Sun Angle":
            instance.actual_instance = SunAngleConstraint.from_json(json_str)
            return instance

        # check if data type is `AltAzConstraint`
        if _data_type == "AltAzConstraint":
            instance.actual_instance = AltAzConstraint.from_json(json_str)
            return instance

        # check if data type is `EarthLimbConstraint`
        if _data_type == "EarthLimbConstraint":
            instance.actual_instance = EarthLimbConstraint.from_json(json_str)
            return instance

        # check if data type is `MoonAngleConstraint`
        if _data_type == "MoonAngleConstraint":
            instance.actual_instance = MoonAngleConstraint.from_json(json_str)
            return instance

        # check if data type is `SAAPolygonConstraint`
        if _data_type == "SAAPolygonConstraint":
            instance.actual_instance = SAAPolygonConstraint.from_json(json_str)
            return instance

        # check if data type is `SunAngleConstraint`
        if _data_type == "SunAngleConstraint":
            instance.actual_instance = SunAngleConstraint.from_json(json_str)
            return instance

        # deserialize data into EarthLimbConstraint
        try:
            instance.actual_instance = EarthLimbConstraint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into MoonAngleConstraint
        try:
            instance.actual_instance = MoonAngleConstraint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into SunAngleConstraint
        try:
            instance.actual_instance = SunAngleConstraint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into SAAPolygonConstraint
        try:
            instance.actual_instance = SAAPolygonConstraint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AltAzConstraint
        try:
            instance.actual_instance = AltAzConstraint.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into InstrumentConstraintsInner with oneOf schemas: AltAzConstraint, EarthLimbConstraint, MoonAngleConstraint, SAAPolygonConstraint, SunAngleConstraint. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into InstrumentConstraintsInner with oneOf schemas: AltAzConstraint, EarthLimbConstraint, MoonAngleConstraint, SAAPolygonConstraint, SunAngleConstraint. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AltAzConstraint, EarthLimbConstraint, MoonAngleConstraint, SAAPolygonConstraint, SunAngleConstraint]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





# coding: utf-8

# flake8: noqa

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "AuthApi",
    "FilterApi",
    "GroupApi",
    "GroupInviteApi",
    "GroupRoleApi",
    "InstrumentApi",
    "InternalApi",
    "ObservationApi",
    "ObservatoryApi",
    "PermissionApi",
    "ScheduleApi",
    "ServiceAccountApi",
    "TLEApi",
    "TelescopeApi",
    "ToolsApi",
    "UserApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "AccessTokenResponse",
    "AcrossServerRoutesV1GroupInviteSchemasGroupInvite",
    "AcrossServerRoutesV1GroupRoleSchemasGroupRole",
    "AcrossServerRoutesV1GroupRoleSchemasServiceAccount",
    "AcrossServerRoutesV1GroupRoleSchemasUser",
    "AcrossServerRoutesV1GroupSchemasGroup",
    "AcrossServerRoutesV1GroupSchemasUser",
    "AcrossServerRoutesV1UserSchemasGroup",
    "AcrossServerRoutesV1UserSchemasGroupInvite",
    "AcrossServerRoutesV1UserSchemasGroupRole",
    "AcrossServerRoutesV1UserSchemasUser",
    "AcrossServerRoutesV1UserServiceAccountSchemasServiceAccount",
    "AltAzConstraint",
    "Bandpass",
    "BandpassType",
    "ConstrainedDate",
    "ConstraintReason",
    "ConstraintType",
    "Coordinate",
    "DateRange",
    "DepthUnit",
    "EarthLimbConstraint",
    "EnergyBandpass",
    "EnergyUnit",
    "EphemerisType",
    "Filter",
    "FrequencyBandpass",
    "FrequencyUnit",
    "GrantType",
    "GroundParameters",
    "GroupInviteCreate",
    "GroupInviteGroupDetails",
    "GroupRead",
    "GroupRoleCreate",
    "GroupRoleRead",
    "HTTPValidationError",
    "IDNameSchema",
    "IVOAObsCategory",
    "IVOAObsTrackingType",
    "Instrument",
    "InstrumentConstraintsInner",
    "JPLParameters",
    "JointVisibilityResult",
    "MoonAngleConstraint",
    "NullableDateRange",
    "Observation",
    "ObservationCreate",
    "ObservationStatus",
    "ObservationType",
    "Observatory",
    "ObservatoryEphemerisType",
    "ObservatoryType",
    "PageObservation",
    "PageSchedule",
    "Parameters",
    "Permission",
    "Point",
    "RoleBase",
    "SAAPolygonConstraint",
    "SPICEParameters",
    "Schedule",
    "ScheduleCadence",
    "ScheduleCreate",
    "ScheduleCreateMany",
    "ScheduleFidelity",
    "ScheduleStatus",
    "ServiceAccountCreate",
    "ServiceAccountSecret",
    "ServiceAccountUpdate",
    "SunAngleConstraint",
    "SystemServiceAccount",
    "TLE",
    "TLECreate",
    "TLEParameters",
    "Telescope",
    "TelescopeInstrument",
    "UnitValue",
    "UserCreate",
    "UserInfo",
    "UserUpdate",
    "ValidationError",
    "ValidationErrorLocInner",
    "VisibilityResult",
    "VisibilityType",
    "VisibilityWindow",
    "WavelengthBandpass",
    "WavelengthUnit",
    "Window",
]

# import apis into sdk package
from across.sdk.v1.api.auth_api import AuthApi as AuthApi
from across.sdk.v1.api.filter_api import FilterApi as FilterApi
from across.sdk.v1.api.group_api import GroupApi as GroupApi
from across.sdk.v1.api.group_invite_api import GroupInviteApi as GroupInviteApi
from across.sdk.v1.api.group_role_api import GroupRoleApi as GroupRoleApi
from across.sdk.v1.api.instrument_api import InstrumentApi as InstrumentApi
from across.sdk.v1.api.internal_api import InternalApi as InternalApi
from across.sdk.v1.api.observation_api import ObservationApi as ObservationApi
from across.sdk.v1.api.observatory_api import ObservatoryApi as ObservatoryApi
from across.sdk.v1.api.permission_api import PermissionApi as PermissionApi
from across.sdk.v1.api.schedule_api import ScheduleApi as ScheduleApi
from across.sdk.v1.api.service_account_api import ServiceAccountApi as ServiceAccountApi
from across.sdk.v1.api.tle_api import TLEApi as TLEApi
from across.sdk.v1.api.telescope_api import TelescopeApi as TelescopeApi
from across.sdk.v1.api.tools_api import ToolsApi as ToolsApi
from across.sdk.v1.api.user_api import UserApi as UserApi

# import ApiClient
from across.sdk.v1.api_response import ApiResponse as ApiResponse
from across.sdk.v1.api_client import ApiClient as ApiClient
from across.sdk.v1.configuration import Configuration as Configuration
from across.sdk.v1.exceptions import OpenApiException as OpenApiException
from across.sdk.v1.exceptions import ApiTypeError as ApiTypeError
from across.sdk.v1.exceptions import ApiValueError as ApiValueError
from across.sdk.v1.exceptions import ApiKeyError as ApiKeyError
from across.sdk.v1.exceptions import ApiAttributeError as ApiAttributeError
from across.sdk.v1.exceptions import ApiException as ApiException

# import models into sdk package
from across.sdk.v1.models.access_token_response import AccessTokenResponse as AccessTokenResponse
from across.sdk.v1.models.across_server_routes_v1_group_invite_schemas_group_invite import AcrossServerRoutesV1GroupInviteSchemasGroupInvite as AcrossServerRoutesV1GroupInviteSchemasGroupInvite
from across.sdk.v1.models.across_server_routes_v1_group_role_schemas_group_role import AcrossServerRoutesV1GroupRoleSchemasGroupRole as AcrossServerRoutesV1GroupRoleSchemasGroupRole
from across.sdk.v1.models.across_server_routes_v1_group_role_schemas_service_account import AcrossServerRoutesV1GroupRoleSchemasServiceAccount as AcrossServerRoutesV1GroupRoleSchemasServiceAccount
from across.sdk.v1.models.across_server_routes_v1_group_role_schemas_user import AcrossServerRoutesV1GroupRoleSchemasUser as AcrossServerRoutesV1GroupRoleSchemasUser
from across.sdk.v1.models.across_server_routes_v1_group_schemas_group import AcrossServerRoutesV1GroupSchemasGroup as AcrossServerRoutesV1GroupSchemasGroup
from across.sdk.v1.models.across_server_routes_v1_group_schemas_user import AcrossServerRoutesV1GroupSchemasUser as AcrossServerRoutesV1GroupSchemasUser
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group import AcrossServerRoutesV1UserSchemasGroup as AcrossServerRoutesV1UserSchemasGroup
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group_invite import AcrossServerRoutesV1UserSchemasGroupInvite as AcrossServerRoutesV1UserSchemasGroupInvite
from across.sdk.v1.models.across_server_routes_v1_user_schemas_group_role import AcrossServerRoutesV1UserSchemasGroupRole as AcrossServerRoutesV1UserSchemasGroupRole
from across.sdk.v1.models.across_server_routes_v1_user_schemas_user import AcrossServerRoutesV1UserSchemasUser as AcrossServerRoutesV1UserSchemasUser
from across.sdk.v1.models.across_server_routes_v1_user_service_account_schemas_service_account import AcrossServerRoutesV1UserServiceAccountSchemasServiceAccount as AcrossServerRoutesV1UserServiceAccountSchemasServiceAccount
from across.sdk.v1.models.alt_az_constraint import AltAzConstraint as AltAzConstraint
from across.sdk.v1.models.bandpass import Bandpass as Bandpass
from across.sdk.v1.models.bandpass_type import BandpassType as BandpassType
from across.sdk.v1.models.constrained_date import ConstrainedDate as ConstrainedDate
from across.sdk.v1.models.constraint_reason import ConstraintReason as ConstraintReason
from across.sdk.v1.models.constraint_type import ConstraintType as ConstraintType
from across.sdk.v1.models.coordinate import Coordinate as Coordinate
from across.sdk.v1.models.date_range import DateRange as DateRange
from across.sdk.v1.models.depth_unit import DepthUnit as DepthUnit
from across.sdk.v1.models.earth_limb_constraint import EarthLimbConstraint as EarthLimbConstraint
from across.sdk.v1.models.energy_bandpass import EnergyBandpass as EnergyBandpass
from across.sdk.v1.models.energy_unit import EnergyUnit as EnergyUnit
from across.sdk.v1.models.ephemeris_type import EphemerisType as EphemerisType
from across.sdk.v1.models.filter import Filter as Filter
from across.sdk.v1.models.frequency_bandpass import FrequencyBandpass as FrequencyBandpass
from across.sdk.v1.models.frequency_unit import FrequencyUnit as FrequencyUnit
from across.sdk.v1.models.grant_type import GrantType as GrantType
from across.sdk.v1.models.ground_parameters import GroundParameters as GroundParameters
from across.sdk.v1.models.group_invite_create import GroupInviteCreate as GroupInviteCreate
from across.sdk.v1.models.group_invite_group_details import GroupInviteGroupDetails as GroupInviteGroupDetails
from across.sdk.v1.models.group_read import GroupRead as GroupRead
from across.sdk.v1.models.group_role_create import GroupRoleCreate as GroupRoleCreate
from across.sdk.v1.models.group_role_read import GroupRoleRead as GroupRoleRead
from across.sdk.v1.models.http_validation_error import HTTPValidationError as HTTPValidationError
from across.sdk.v1.models.id_name_schema import IDNameSchema as IDNameSchema
from across.sdk.v1.models.ivoa_obs_category import IVOAObsCategory as IVOAObsCategory
from across.sdk.v1.models.ivoa_obs_tracking_type import IVOAObsTrackingType as IVOAObsTrackingType
from across.sdk.v1.models.instrument import Instrument as Instrument
from across.sdk.v1.models.instrument_constraints_inner import InstrumentConstraintsInner as InstrumentConstraintsInner
from across.sdk.v1.models.jpl_parameters import JPLParameters as JPLParameters
from across.sdk.v1.models.joint_visibility_result import JointVisibilityResult as JointVisibilityResult
from across.sdk.v1.models.moon_angle_constraint import MoonAngleConstraint as MoonAngleConstraint
from across.sdk.v1.models.nullable_date_range import NullableDateRange as NullableDateRange
from across.sdk.v1.models.observation import Observation as Observation
from across.sdk.v1.models.observation_create import ObservationCreate as ObservationCreate
from across.sdk.v1.models.observation_status import ObservationStatus as ObservationStatus
from across.sdk.v1.models.observation_type import ObservationType as ObservationType
from across.sdk.v1.models.observatory import Observatory as Observatory
from across.sdk.v1.models.observatory_ephemeris_type import ObservatoryEphemerisType as ObservatoryEphemerisType
from across.sdk.v1.models.observatory_type import ObservatoryType as ObservatoryType
from across.sdk.v1.models.page_observation import PageObservation as PageObservation
from across.sdk.v1.models.page_schedule import PageSchedule as PageSchedule
from across.sdk.v1.models.parameters import Parameters as Parameters
from across.sdk.v1.models.permission import Permission as Permission
from across.sdk.v1.models.point import Point as Point
from across.sdk.v1.models.role_base import RoleBase as RoleBase
from across.sdk.v1.models.saa_polygon_constraint import SAAPolygonConstraint as SAAPolygonConstraint
from across.sdk.v1.models.spice_parameters import SPICEParameters as SPICEParameters
from across.sdk.v1.models.schedule import Schedule as Schedule
from across.sdk.v1.models.schedule_cadence import ScheduleCadence as ScheduleCadence
from across.sdk.v1.models.schedule_create import ScheduleCreate as ScheduleCreate
from across.sdk.v1.models.schedule_create_many import ScheduleCreateMany as ScheduleCreateMany
from across.sdk.v1.models.schedule_fidelity import ScheduleFidelity as ScheduleFidelity
from across.sdk.v1.models.schedule_status import ScheduleStatus as ScheduleStatus
from across.sdk.v1.models.service_account_create import ServiceAccountCreate as ServiceAccountCreate
from across.sdk.v1.models.service_account_secret import ServiceAccountSecret as ServiceAccountSecret
from across.sdk.v1.models.service_account_update import ServiceAccountUpdate as ServiceAccountUpdate
from across.sdk.v1.models.sun_angle_constraint import SunAngleConstraint as SunAngleConstraint
from across.sdk.v1.models.system_service_account import SystemServiceAccount as SystemServiceAccount
from across.sdk.v1.models.tle import TLE as TLE
from across.sdk.v1.models.tle_create import TLECreate as TLECreate
from across.sdk.v1.models.tle_parameters import TLEParameters as TLEParameters
from across.sdk.v1.models.telescope import Telescope as Telescope
from across.sdk.v1.models.telescope_instrument import TelescopeInstrument as TelescopeInstrument
from across.sdk.v1.models.unit_value import UnitValue as UnitValue
from across.sdk.v1.models.user_create import UserCreate as UserCreate
from across.sdk.v1.models.user_info import UserInfo as UserInfo
from across.sdk.v1.models.user_update import UserUpdate as UserUpdate
from across.sdk.v1.models.validation_error import ValidationError as ValidationError
from across.sdk.v1.models.validation_error_loc_inner import ValidationErrorLocInner as ValidationErrorLocInner
from across.sdk.v1.models.visibility_result import VisibilityResult as VisibilityResult
from across.sdk.v1.models.visibility_type import VisibilityType as VisibilityType
from across.sdk.v1.models.visibility_window import VisibilityWindow as VisibilityWindow
from across.sdk.v1.models.wavelength_bandpass import WavelengthBandpass as WavelengthBandpass
from across.sdk.v1.models.wavelength_unit import WavelengthUnit as WavelengthUnit
from across.sdk.v1.models.window import Window as Window



# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import List, Optional, Union
from typing_extensions import Annotated
from across.sdk.v1.models.joint_visibility_result import JointVisibilityResult
from across.sdk.v1.models.visibility_result import VisibilityResult

from across.sdk.v1.api_client import ApiClient, RequestSerialized
from across.sdk.v1.api_response import ApiResponse
from across.sdk.v1.rest import RESTResponseType


class ToolsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def calculate_joint_windows_tools_visibility_calculator_windows_get(
        self,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        instrument_ids: List[Optional[StrictStr]],
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> JointVisibilityResult:
        """Calculated Joint Visibility Windows

        Calculate joint visibility windows between instruments for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param instrument_ids: (required)
        :type instrument_ids: List[Optional[str]]
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_joint_windows_tools_visibility_calculator_windows_get_serialize(
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            instrument_ids=instrument_ids,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "JointVisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def calculate_joint_windows_tools_visibility_calculator_windows_get_with_http_info(
        self,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        instrument_ids: List[Optional[StrictStr]],
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[JointVisibilityResult]:
        """Calculated Joint Visibility Windows

        Calculate joint visibility windows between instruments for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param instrument_ids: (required)
        :type instrument_ids: List[Optional[str]]
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_joint_windows_tools_visibility_calculator_windows_get_serialize(
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            instrument_ids=instrument_ids,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "JointVisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def calculate_joint_windows_tools_visibility_calculator_windows_get_without_preload_content(
        self,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        instrument_ids: List[Optional[StrictStr]],
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Calculated Joint Visibility Windows

        Calculate joint visibility windows between instruments for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param instrument_ids: (required)
        :type instrument_ids: List[Optional[str]]
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_joint_windows_tools_visibility_calculator_windows_get_serialize(
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            instrument_ids=instrument_ids,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "JointVisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _calculate_joint_windows_tools_visibility_calculator_windows_get_serialize(
        self,
        ra,
        dec,
        date_range_begin,
        date_range_end,
        instrument_ids,
        hi_res,
        min_visibility_duration,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'instrument_ids': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if ra is not None:
            
            _query_params.append(('ra', ra))
            
        if dec is not None:
            
            _query_params.append(('dec', dec))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if hi_res is not None:
            
            _query_params.append(('hi_res', hi_res))
            
        if min_visibility_duration is not None:
            
            _query_params.append(('min_visibility_duration', min_visibility_duration))
            
        if instrument_ids is not None:
            
            _query_params.append(('instrument_ids', instrument_ids))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/tools/visibility-calculator/windows/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def calculate_joint_windows_tools_visibility_calculator_windows_get_0(
        self,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        instrument_ids: List[Optional[StrictStr]],
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> JointVisibilityResult:
        """Calculated Joint Visibility Windows

        Calculate joint visibility windows between instruments for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param instrument_ids: (required)
        :type instrument_ids: List[Optional[str]]
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_joint_windows_tools_visibility_calculator_windows_get_0_serialize(
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            instrument_ids=instrument_ids,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "JointVisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def calculate_joint_windows_tools_visibility_calculator_windows_get_0_with_http_info(
        self,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        instrument_ids: List[Optional[StrictStr]],
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[JointVisibilityResult]:
        """Calculated Joint Visibility Windows

        Calculate joint visibility windows between instruments for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param instrument_ids: (required)
        :type instrument_ids: List[Optional[str]]
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_joint_windows_tools_visibility_calculator_windows_get_0_serialize(
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            instrument_ids=instrument_ids,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "JointVisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def calculate_joint_windows_tools_visibility_calculator_windows_get_0_without_preload_content(
        self,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        instrument_ids: List[Optional[StrictStr]],
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Calculated Joint Visibility Windows

        Calculate joint visibility windows between instruments for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param instrument_ids: (required)
        :type instrument_ids: List[Optional[str]]
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_joint_windows_tools_visibility_calculator_windows_get_0_serialize(
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            instrument_ids=instrument_ids,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "JointVisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _calculate_joint_windows_tools_visibility_calculator_windows_get_0_serialize(
        self,
        ra,
        dec,
        date_range_begin,
        date_range_end,
        instrument_ids,
        hi_res,
        min_visibility_duration,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'instrument_ids': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if ra is not None:
            
            _query_params.append(('ra', ra))
            
        if dec is not None:
            
            _query_params.append(('dec', dec))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if hi_res is not None:
            
            _query_params.append(('hi_res', hi_res))
            
        if min_visibility_duration is not None:
            
            _query_params.append(('min_visibility_duration', min_visibility_duration))
            
        if instrument_ids is not None:
            
            _query_params.append(('instrument_ids', instrument_ids))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/tools/visibility-calculator/windows/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def calculate_windows_tools_visibility_calculator_windows_instrument_id_get(
        self,
        instrument_id: StrictStr,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> VisibilityResult:
        """Calculated Visibility Windows

        Calculate visibility windows of an instrument for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param instrument_id: (required)
        :type instrument_id: str
        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_windows_tools_visibility_calculator_windows_instrument_id_get_serialize(
            instrument_id=instrument_id,
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "VisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def calculate_windows_tools_visibility_calculator_windows_instrument_id_get_with_http_info(
        self,
        instrument_id: StrictStr,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[VisibilityResult]:
        """Calculated Visibility Windows

        Calculate visibility windows of an instrument for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param instrument_id: (required)
        :type instrument_id: str
        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_windows_tools_visibility_calculator_windows_instrument_id_get_serialize(
            instrument_id=instrument_id,
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "VisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def calculate_windows_tools_visibility_calculator_windows_instrument_id_get_without_preload_content(
        self,
        instrument_id: StrictStr,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Calculated Visibility Windows

        Calculate visibility windows of an instrument for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param instrument_id: (required)
        :type instrument_id: str
        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_windows_tools_visibility_calculator_windows_instrument_id_get_serialize(
            instrument_id=instrument_id,
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "VisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _calculate_windows_tools_visibility_calculator_windows_instrument_id_get_serialize(
        self,
        instrument_id,
        ra,
        dec,
        date_range_begin,
        date_range_end,
        hi_res,
        min_visibility_duration,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instrument_id is not None:
            _path_params['instrument_id'] = instrument_id
        # process the query parameters
        if ra is not None:
            
            _query_params.append(('ra', ra))
            
        if dec is not None:
            
            _query_params.append(('dec', dec))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if hi_res is not None:
            
            _query_params.append(('hi_res', hi_res))
            
        if min_visibility_duration is not None:
            
            _query_params.append(('min_visibility_duration', min_visibility_duration))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/tools/visibility-calculator/windows/{instrument_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0(
        self,
        instrument_id: StrictStr,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> VisibilityResult:
        """Calculated Visibility Windows

        Calculate visibility windows of an instrument for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param instrument_id: (required)
        :type instrument_id: str
        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0_serialize(
            instrument_id=instrument_id,
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "VisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0_with_http_info(
        self,
        instrument_id: StrictStr,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[VisibilityResult]:
        """Calculated Visibility Windows

        Calculate visibility windows of an instrument for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param instrument_id: (required)
        :type instrument_id: str
        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0_serialize(
            instrument_id=instrument_id,
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "VisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0_without_preload_content(
        self,
        instrument_id: StrictStr,
        ra: Union[Annotated[float, Field(le=360.0, strict=True, ge=0.0)], Annotated[int, Field(le=360, strict=True, ge=0)]],
        dec: Union[Annotated[float, Field(le=90.0, strict=True, ge=-90.0)], Annotated[int, Field(le=90, strict=True, ge=-90)]],
        date_range_begin: datetime,
        date_range_end: datetime,
        hi_res: Optional[StrictBool] = None,
        min_visibility_duration: Optional[StrictInt] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Calculated Visibility Windows

        Calculate visibility windows of an instrument for a given observation coordinate    Use GET /telescope/ to retrieve a list of telescopes and their associated instruments    WARNING: This is a long running process and is liable to timeout after a strict 60 second execution limit    If you experience issues retrieving results, please scope your date range to be a smaller window or set `hi_res` to `false`

        :param instrument_id: (required)
        :type instrument_id: str
        :param ra: (required)
        :type ra: float
        :param dec: (required)
        :type dec: float
        :param date_range_begin: (required)
        :type date_range_begin: datetime
        :param date_range_end: (required)
        :type date_range_end: datetime
        :param hi_res:
        :type hi_res: bool
        :param min_visibility_duration:
        :type min_visibility_duration: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0_serialize(
            instrument_id=instrument_id,
            ra=ra,
            dec=dec,
            date_range_begin=date_range_begin,
            date_range_end=date_range_end,
            hi_res=hi_res,
            min_visibility_duration=min_visibility_duration,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "VisibilityResult",
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _calculate_windows_tools_visibility_calculator_windows_instrument_id_get_0_serialize(
        self,
        instrument_id,
        ra,
        dec,
        date_range_begin,
        date_range_end,
        hi_res,
        min_visibility_duration,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if instrument_id is not None:
            _path_params['instrument_id'] = instrument_id
        # process the query parameters
        if ra is not None:
            
            _query_params.append(('ra', ra))
            
        if dec is not None:
            
            _query_params.append(('dec', dec))
            
        if date_range_begin is not None:
            if isinstance(date_range_begin, datetime):
                _query_params.append(
                    (
                        'date_range_begin',
                        date_range_begin.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_begin', date_range_begin))
            
        if date_range_end is not None:
            if isinstance(date_range_end, datetime):
                _query_params.append(
                    (
                        'date_range_end',
                        date_range_end.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('date_range_end', date_range_end))
            
        if hi_res is not None:
            
            _query_params.append(('hi_res', hi_res))
            
        if min_visibility_duration is not None:
            
            _query_params.append(('min_visibility_duration', min_visibility_duration))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/tools/visibility-calculator/windows/{instrument_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    ACROSS Server

    Server providing tools and utilities for various NASA missions to aid in coordination of large observation efforts.

    The version of the OpenAPI document: 1.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class MoonAngleConstraint(BaseModel):
    """
    For a given Moon avoidance angle, is a given coordinate inside this constraint?  Parameters ---------- min_angle     The minimum angle from the Moon that the spacecraft can point.  Methods ------- __call__(coord, ephemeris, moon_radius_angle=None)     Checks if a given coordinate is inside the constraint.
    """ # noqa: E501
    short_name: Optional[StrictStr] = 'Moon'
    name: Optional[StrictStr] = 'Moon Angle'
    min_angle: Optional[Union[Annotated[float, Field(le=180.0, strict=True, ge=0.0)], Annotated[int, Field(le=180, strict=True, ge=0)]]] = None
    max_angle: Optional[Union[Annotated[float, Field(le=180.0, strict=True, ge=0.0)], Annotated[int, Field(le=180, strict=True, ge=0)]]] = None
    __properties: ClassVar[List[str]] = ["short_name", "name", "min_angle", "max_angle"]

    @field_validator('short_name')
    def short_name_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Moon']):
            raise ValueError("must be one of enum values ('Moon')")
        return value

    @field_validator('name')
    def name_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Moon Angle']):
            raise ValueError("must be one of enum values ('Moon Angle')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MoonAngleConstraint from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if min_angle (nullable) is None
        # and model_fields_set contains the field
        if self.min_angle is None and "min_angle" in self.model_fields_set:
            _dict['min_angle'] = None

        # set to None if max_angle (nullable) is None
        # and model_fields_set contains the field
        if self.max_angle is None and "max_angle" in self.model_fields_set:
            _dict['max_angle'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MoonAngleConstraint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "short_name": obj.get("short_name") if obj.get("short_name") is not None else 'Moon',
            "name": obj.get("name") if obj.get("name") is not None else 'Moon Angle',
            "min_angle": obj.get("min_angle"),
            "max_angle": obj.get("max_angle")
        })
        return _obj





from abc import ABC, abstractmethod


class CredentialStorage(ABC):
    @property
    @abstractmethod
    def days_before_exp(self) -> int: ...

    @abstractmethod
    def id(self, force: bool = False) -> str: ...

    @abstractmethod
    def secret(self, force: bool = False) -> str: ...

    @abstractmethod
    def update_key(self, key: str) -> None: ...
